@@ -1,11 +1,12 @@
 from unittest.mock import patch
 
 from django.contrib.auth.models import User
+from django.test import TestCase
 from django.urls import reverse
 from django_webtest import WebTest
 
-from ..models import DiscordWebhook, NotificationRule, Timer
-from . import LoadTestDataMixin
+from ..models import DiscordWebhook, NotificationRule, StagingSystem, Timer
+from . import LoadTestDataMixin, create_fake_staging_system
 
 
 @patch("structuretimers.models.STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
@@ -23,26 +24,26 @@ class TestNotificationRuleChangeList(LoadTestDataMixin, WebTest):
     @patch("structuretimers.models.STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
     def setUp(self) -> None:
         NotificationRule.objects.create(
-            trigger=NotificationRule.TRIGGER_SCHEDULED_TIME_REACHED,
+            trigger=NotificationRule.Trigger.SCHEDULED_TIME_REACHED,
             scheduled_time=NotificationRule.MINUTES_10,
             webhook=self.webhook,
         )
         NotificationRule.objects.create(
-            trigger=NotificationRule.TRIGGER_SCHEDULED_TIME_REACHED,
+            trigger=NotificationRule.Trigger.SCHEDULED_TIME_REACHED,
             scheduled_time=NotificationRule.MINUTES_10,
-            require_timer_types=[Timer.TYPE_ARMOR],
+            require_timer_types=[Timer.Type.ARMOR],
             webhook=self.webhook,
         )
         rule = NotificationRule.objects.create(
-            trigger=NotificationRule.TRIGGER_SCHEDULED_TIME_REACHED,
+            trigger=NotificationRule.Trigger.SCHEDULED_TIME_REACHED,
             scheduled_time=NotificationRule.MINUTES_10,
             webhook=self.webhook,
         )
         rule.require_corporations.add(self.corporation_1)
         NotificationRule.objects.create(
-            trigger=NotificationRule.TRIGGER_SCHEDULED_TIME_REACHED,
+            trigger=NotificationRule.Trigger.SCHEDULED_TIME_REACHED,
             scheduled_time=NotificationRule.MINUTES_10,
-            is_important=NotificationRule.CLAUSE_EXCLUDED,
+            is_important=NotificationRule.Clause.EXCLUDED,
             webhook=self.webhook,
         )
 
@@ -73,9 +74,6 @@ class TestNotificationRuleValidations(LoadTestDataMixin, WebTest):
             "admin:structuretimers_notificationrule_changelist"
         )
 
-    def setUp(self) -> None:
-        NotificationRule.objects.all().delete()
-
     def _open_page(self) -> object:
         # login
         self.app.set_user(self.user)
@@ -84,7 +82,7 @@ class TestNotificationRuleValidations(LoadTestDataMixin, WebTest):
         add_page = self.app.get(self.url_add)
         self.assertEqual(add_page.status_code, 200)
         form = add_page.form
-        form["trigger"] = NotificationRule.TRIGGER_SCHEDULED_TIME_REACHED
+        form["trigger"] = NotificationRule.Trigger.SCHEDULED_TIME_REACHED
         form["scheduled_time"] = NotificationRule.MINUTES_10
         form["webhook"] = self.webhook.pk
         return form
@@ -100,8 +98,8 @@ class TestNotificationRuleValidations(LoadTestDataMixin, WebTest):
 
     def test_can_not_have_same_options_timer_types(self):
         form = self._open_page()
-        form["require_timer_types"] = [Timer.TYPE_ANCHORING, Timer.TYPE_HULL]
-        form["exclude_timer_types"] = [Timer.TYPE_ANCHORING, Timer.TYPE_ARMOR]
+        form["require_timer_types"] = [Timer.Type.ANCHORING, Timer.Type.HULL]
+        form["exclude_timer_types"] = [Timer.Type.ANCHORING, Timer.Type.ARMOR]
         response = form.submit()
 
         # assert results
@@ -111,8 +109,8 @@ class TestNotificationRuleValidations(LoadTestDataMixin, WebTest):
 
     def test_can_not_have_same_options_objectives(self):
         form = self._open_page()
-        form["require_objectives"] = [Timer.OBJECTIVE_FRIENDLY, Timer.OBJECTIVE_HOSTILE]
-        form["exclude_objectives"] = [Timer.OBJECTIVE_FRIENDLY, Timer.OBJECTIVE_NEUTRAL]
+        form["require_objectives"] = [Timer.Objective.FRIENDLY, Timer.Objective.HOSTILE]
+        form["exclude_objectives"] = [Timer.Objective.FRIENDLY, Timer.Objective.NEUTRAL]
         response = form.submit()
 
         # assert results
@@ -122,8 +120,8 @@ class TestNotificationRuleValidations(LoadTestDataMixin, WebTest):
 
     def test_can_not_have_same_options_visibility(self):
         form = self._open_page()
-        form["require_visibility"] = [Timer.VISIBILITY_CORPORATION]
-        form["exclude_visibility"] = [Timer.VISIBILITY_CORPORATION]
+        form["require_visibility"] = [Timer.Visibility.CORPORATION]
+        form["exclude_visibility"] = [Timer.Visibility.CORPORATION]
         response = form.submit()
 
         # assert results
@@ -152,3 +150,41 @@ class TestNotificationRuleValidations(LoadTestDataMixin, WebTest):
         self.assertEqual(response.status_code, 200)
         self.assertIn("Please correct the error below", response.text)
         self.assertEqual(NotificationRule.objects.count(), 0)
+
+
+class TestStagingSystemAdmin(LoadTestDataMixin, TestCase):
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        cls.user = User.objects.create_superuser("Bruce Wayne")
+        cls.url_add = reverse("admin:structuretimers_stagingsystem_add")
+
+    def test_should_create_new_staging_system(self):
+        # given
+        self.client.force_login(self.user)
+        # when
+        res = self.client.post(
+            self.url_add, data={"eve_solar_system": self.system_abune.pk}
+        )
+        # then
+        self.assertEqual(res.status_code, 302)
+        self.assertEqual(StagingSystem.objects.count(), 1)
+        obj = StagingSystem.objects.first()
+        self.assertEqual(obj.eve_solar_system, self.system_abune)
+        self.assertFalse(obj.is_main)
+
+    def test_should_ensure_only_one_obj_is_main(self):
+        # given
+        self.client.force_login(self.user)
+        create_fake_staging_system(eve_solar_system=self.system_enaluri, is_main=True)
+        # when
+        res = self.client.post(
+            self.url_add,
+            data={"eve_solar_system": self.system_abune.pk, "is_main": True},
+        )
+        # then
+        self.assertEqual(res.status_code, 302)
+        self.assertEqual(
+            StagingSystem.objects.filter(is_main=True).get().eve_solar_system,
+            self.system_abune,
+        )
