@@ -1,5 +1,5 @@
 from datetime import timedelta
-from unittest.mock import patch
+from unittest.mock import Mock, patch
 
 from requests.exceptions import ConnectionError as NewConnectionError
 from requests.exceptions import HTTPError
@@ -14,9 +14,18 @@ from allianceauth.tests.auth_utils import AuthUtils
 
 from ..models import DiscordWebhook, Timer
 from ..tasks import send_test_message_to_webhook
-from . import LoadTestDataMixin, add_permission_to_user_by_name, create_test_user
+from . import (
+    LoadTestDataMixin,
+    add_permission_to_user_by_name,
+    create_fake_timer,
+    create_test_user,
+)
 from .testdata import test_data_filename, test_image_filename
 
+MODELS_PATH = "structuretimers.models"
+FORMS_PATH = "structuretimers.forms"
+TASKS_PATH = "structuretimers.tasks"
+
 
 def bytes_from_file(filename, chunksize=8192):
     with open(filename, "rb") as f:
@@ -29,7 +38,8 @@ def bytes_from_file(filename, chunksize=8192):
                 break
 
 
-@patch("structuretimers.models.STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
+@patch(MODELS_PATH + "._task_calc_timer_distances_for_all_staging_systems", Mock())
+@patch(MODELS_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
 class TestUI(LoadTestDataMixin, WebTest):
     @classmethod
     def setUpClass(cls):
@@ -44,9 +54,9 @@ class TestUI(LoadTestDataMixin, WebTest):
             "structuretimers.create_timer", cls.user_2
         )
 
-    @patch("structuretimers.models.STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
+    @patch(MODELS_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
     def setUp(self) -> None:
-        self.timer_1 = Timer.objects.create(
+        self.timer_1 = create_fake_timer(
             structure_name="Timer 1",
             date=now() + timedelta(hours=4),
             eve_character=self.character_2,
@@ -55,7 +65,7 @@ class TestUI(LoadTestDataMixin, WebTest):
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
         )
-        self.timer_2 = Timer.objects.create(
+        self.timer_2 = create_fake_timer(
             structure_name="Timer 2",
             date=now() - timedelta(hours=8),
             eve_character=self.character_2,
@@ -64,7 +74,7 @@ class TestUI(LoadTestDataMixin, WebTest):
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
         )
-        self.timer_3 = Timer.objects.create(
+        self.timer_3 = create_fake_timer(
             structure_name="Timer 3",
             date=now() - timedelta(hours=8),
             eve_character=self.character_2,
@@ -73,9 +83,8 @@ class TestUI(LoadTestDataMixin, WebTest):
             eve_solar_system=self.system_enaluri,
             structure_type=self.type_astrahus,
         )
-        self.timer_3.save()
 
-    def test_add_new_timer(self):
+    def test_should_add_new_timer(self):
         """
         when user has permissions
         then he can create a new timer
@@ -94,19 +103,24 @@ class TestUI(LoadTestDataMixin, WebTest):
 
         # user enters data and clicks create
         form = add_timer.forms["add-timer-form"]
-        form["days_left"] = 1
         form["structure_name"] = "Timer 4"
         form["eve_solar_system_2"].force_value([str(self.system_abune.id)])
         form["structure_type_2"].force_value([str(self.type_astrahus.id)])
+        form["timer_type"] = Timer.Type.ANCHORING
         form["days_left"] = 1
         form["hours_left"] = 2
         form["minutes_left"] = 3
         response = form.submit()
 
         # assert results
+        timer_date = now() + timedelta(days=1, hours=2, minutes=3)
         self.assertEqual(response.status_code, 302)
         self.assertEqual(response.url, reverse("structuretimers:timer_list"))
-        self.assertTrue(Timer.objects.filter(structure_name="Timer 4").exists())
+        obj = Timer.objects.get(structure_name="Timer 4")
+        self.assertEqual(obj.eve_solar_system, self.system_abune)
+        self.assertEqual(obj.structure_type, self.type_astrahus)
+        self.assertEqual(obj.timer_type, Timer.Type.ANCHORING)
+        self.assertAlmostEqual(obj.date, timer_date, delta=timedelta(seconds=10))
 
     def test_add_new_timer_without_permission(self):
         """
@@ -195,7 +209,7 @@ class TestUI(LoadTestDataMixin, WebTest):
         self.assertEqual(response.status_code, 200)
         self.assertIn("correct the input errors", response.text)
 
-    @patch("structuretimers.forms.requests.get", spec=True)
+    @patch(FORMS_PATH + ".requests.get", spec=True)
     def test_should_show_error_when_image_can_not_be_loaded_1(self, mock_get):
         mock_get.side_effect = NewConnectionError
 
@@ -221,7 +235,7 @@ class TestUI(LoadTestDataMixin, WebTest):
         self.assertEqual(response.status_code, 200)
         self.assertIn("correct the input errors", response.text)
 
-    @patch("structuretimers.forms.requests.get", spec=True)
+    @patch(FORMS_PATH + ".requests.get", spec=True)
     def test_should_show_error_when_image_can_not_be_loaded_2(self, mock_get):
         """
         when user entered invalid day
@@ -252,7 +266,7 @@ class TestUI(LoadTestDataMixin, WebTest):
         self.assertEqual(response.status_code, 200)
         self.assertIn("correct the input errors", response.text)
 
-    @patch("structuretimers.forms.requests.get", spec=True)
+    @patch(FORMS_PATH + ".requests.get", spec=True)
     def test_should_create_timer_with_valid_details_image(self, mock_get):
         image_file = bytearray(bytes_from_file(test_image_filename()))
         mock_get.return_value.content = image_file
@@ -281,7 +295,7 @@ class TestUI(LoadTestDataMixin, WebTest):
         self.assertEqual(response.url, reverse("structuretimers:timer_list"))
         self.assertTrue(Timer.objects.filter(structure_name="Timer 4").exists())
 
-    @patch("structuretimers.forms.requests.get", spec=True)
+    @patch(FORMS_PATH + ".requests.get", spec=True)
     def test_should_not_allow_invalid_link_for_detail_images(self, mock_get):
         """
         when user provides invalid file link
@@ -313,6 +327,24 @@ class TestUI(LoadTestDataMixin, WebTest):
         self.assertEqual(response.status_code, 200)
         self.assertIn("correct the input errors", response.text)
 
+    def test_should_not_allow_moon_mining_type_for_non_mining_structures(self):
+        # given
+        self.app.set_user(self.user_2)
+        add_timer = self.app.get(reverse("structuretimers:add"))
+        # when
+        form = add_timer.forms["add-timer-form"]
+        form["structure_name"] = "Timer 4"
+        form["eve_solar_system_2"].force_value([str(self.system_abune.id)])
+        form["structure_type_2"].force_value([str(self.type_astrahus.id)])
+        form["timer_type"] = Timer.Type.MOONMINING
+        form["days_left"] = 1
+        form["hours_left"] = 2
+        form["minutes_left"] = 3
+        response = form.submit()
+        # then
+        self.assertEqual(response.status_code, 200)
+        self.assertIn("correct the input errors", response.text)
+
     def test_edit_existing_timer(self):
         """
         when user has permissions
@@ -414,7 +446,7 @@ class TestUI(LoadTestDataMixin, WebTest):
         from another corp
         then he is redirected back to dashboard
         """
-        self.timer_3.visibility = Timer.VISIBILITY_CORPORATION
+        self.timer_3.visibility = Timer.Visibility.CORPORATION
         self.timer_3.save()
 
         # login
@@ -528,8 +560,8 @@ class TestUI(LoadTestDataMixin, WebTest):
 
 
 """
-@patch("structuretimers.models.sleep", new=lambda x: x)
-@patch("structuretimers.models.dhooks_lite.Webhook.execute")
+@patch(MODELS_PATH+ ".sleep", new=lambda x: x)
+@patch(MODELS_PATH+ ".dhooks_lite.Webhook.execute")
 class TestSendNotifications(LoadTestDataMixin, TestCase):
     def setUp(self) -> None:
         self.webhook = DiscordWebhook.objects.create(
@@ -539,7 +571,7 @@ class TestSendNotifications(LoadTestDataMixin, TestCase):
         self.rule.webhooks.add(self.webhook)
 
     def test_normal(self, mock_execute):
-        Timer.objects.create(
+        create_fake_timer(
             structure_name="Test_1",
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
@@ -551,9 +583,9 @@ class TestSendNotifications(LoadTestDataMixin, TestCase):
 
 
 @override_settings(CELERY_ALWAYS_EAGER=True)
-@patch("structuretimers.models.sleep", new=lambda x: x)
-@patch("structuretimers.tasks.notify", spec=True)
-@patch("structuretimers.models.dhooks_lite.Webhook.execute", spec=True)
+@patch(MODELS_PATH + ".sleep", new=lambda x: x)
+@patch(TASKS_PATH + ".notify", spec=True)
+@patch(MODELS_PATH + ".dhooks_lite.Webhook.execute", spec=True)
 class TestTestMessageToWebhook(LoadTestDataMixin, TestCase):
     def setUp(self) -> None:
         self.webhook = DiscordWebhook.objects.create(
