@@ -1,34 +1,43 @@
 import json
 from datetime import datetime, timedelta
-from unittest.mock import patch
+from unittest.mock import Mock, patch
 
 import dhooks_lite
 
 from django.core.cache import cache
-from django.test import TestCase
+from django.db import models
+from django.test import TestCase, override_settings
 from django.utils.timezone import now
 
 from allianceauth.eveonline.models import EveAllianceInfo, EveCorporationInfo
 from app_utils.json import JSONDateTimeDecoder
 from app_utils.testing import NoSocketsTestCase
 
+from .. import __title__
 from ..models import (
     DiscordWebhook,
     NotificationRule,
     ScheduledNotification,
+    StagingSystem,
     Timer,
-    models,
+    _task_calc_staging_system,
+)
+from . import (
+    LoadTestDataMixin,
+    add_permission_to_user_by_name,
+    create_fake_staging_system,
+    create_fake_timer,
+    create_test_user,
 )
-from . import LoadTestDataMixin, add_permission_to_user_by_name, create_test_user
 
 MODULE_PATH = "structuretimers.models"
 
 
-class TestTimer(LoadTestDataMixin, TestCase):
+class TestTimer(LoadTestDataMixin, NoSocketsTestCase):
     def test_str(self):
         timer = Timer(
             structure_name="Test",
-            timer_type=Timer.TYPE_ARMOR,
+            timer_type=Timer.Type.ARMOR,
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
             date=datetime(2020, 8, 6, 13, 25),
@@ -38,7 +47,7 @@ class TestTimer(LoadTestDataMixin, TestCase):
 
     def test_structure_display_name_1(self):
         timer = Timer(
-            timer_type=Timer.TYPE_ARMOR,
+            timer_type=Timer.Type.ARMOR,
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
             date=datetime(2020, 8, 6, 13, 25),
@@ -48,7 +57,7 @@ class TestTimer(LoadTestDataMixin, TestCase):
 
     def test_structure_display_name_2(self):
         timer = Timer(
-            timer_type=Timer.TYPE_ARMOR,
+            timer_type=Timer.Type.ARMOR,
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
             location_details="P5-M3",
@@ -60,7 +69,7 @@ class TestTimer(LoadTestDataMixin, TestCase):
     def test_structure_display_name_3(self):
         timer = Timer(
             structure_name="Big Boy",
-            timer_type=Timer.TYPE_ARMOR,
+            timer_type=Timer.Type.ARMOR,
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
             date=datetime(2020, 8, 6, 13, 25),
@@ -72,24 +81,25 @@ class TestTimer(LoadTestDataMixin, TestCase):
         timer = Timer(date=now())
         self.assertEqual(timer.label_type_for_timer_type(), "default")
 
-        timer.timer_type = timer.TYPE_ARMOR
+        timer.timer_type = Timer.Type.ARMOR
         self.assertEqual(timer.label_type_for_timer_type(), "danger")
 
-        timer.timer_type = timer.TYPE_HULL
+        timer.timer_type = Timer.Type.HULL
         self.assertEqual(timer.label_type_for_timer_type(), "danger")
 
     def test_label_type_for_objective(self):
         timer = Timer(date=now())
         self.assertEqual(timer.label_type_for_objective(), "default")
 
-        timer.objective = Timer.OBJECTIVE_HOSTILE
+        timer.objective = Timer.Objective.HOSTILE
         self.assertEqual(timer.label_type_for_objective(), "danger")
 
-        timer.objective = Timer.OBJECTIVE_FRIENDLY
+        timer.objective = Timer.Objective.FRIENDLY
         self.assertEqual(timer.label_type_for_objective(), "primary")
 
 
-class TestTimerSave(LoadTestDataMixin, TestCase):
+@patch(MODULE_PATH + "._task_calc_timer_distances_for_all_staging_systems", Mock())
+class TestTimerSaveXScheduleNotifications(LoadTestDataMixin, NoSocketsTestCase):
     @classmethod
     def setUpClass(cls) -> None:
         super().setUpClass()
@@ -97,20 +107,21 @@ class TestTimerSave(LoadTestDataMixin, TestCase):
             name="Dummy", url="http://www.example.com"
         )
 
-    @patch(MODULE_PATH + ".Timer._import_schedule_notifications_for_timer")
-    def test_schedule_notifications_for_new_timers(self, mock_import_func):
-        timer = Timer.objects.create(
+    @patch(MODULE_PATH + "._task_schedule_notifications_for_timer")
+    def test_schedule_notifications_for_new_timers(self, mock_schedule_notifications):
+        timer = create_fake_timer(
             date=now() + timedelta(hours=4),
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
+            enabled_notifications=True,
         )
-        self.assertTrue(mock_import_func.called)
-        _, kwargs = mock_import_func.return_value.apply_async.call_args
+        self.assertTrue(mock_schedule_notifications.called)
+        _, kwargs = mock_schedule_notifications.return_value.apply_async.call_args
         self.assertEqual(kwargs["kwargs"]["timer_pk"], timer.pk)
 
-    @patch(MODULE_PATH + ".Timer._import_schedule_notifications_for_timer")
+    @patch(MODULE_PATH + "._task_schedule_notifications_for_timer")
     def test_dont_schedule_notifications_for_new_timers_when_turned_off(
-        self, mock_import_func
+        self, mock_schedule_notifications
     ):
         timer = Timer(
             date=now() + timedelta(hours=4),
@@ -118,47 +129,100 @@ class TestTimerSave(LoadTestDataMixin, TestCase):
             structure_type=self.type_astrahus,
         )
         timer.save(disable_notifications=True)
-        self.assertFalse(mock_import_func.called)
+        self.assertFalse(mock_schedule_notifications.called)
 
     def test_schedule_notifications_when_date_changed(self):
         with patch(
-            MODULE_PATH + ".Timer._import_schedule_notifications_for_timer"
-        ) as mock_import_func:
-            timer = Timer.objects.create(
+            MODULE_PATH + "._task_schedule_notifications_for_timer"
+        ) as mock_schedule_notifications:
+            timer = create_fake_timer(
                 date=now() + timedelta(hours=4),
                 eve_solar_system=self.system_abune,
                 structure_type=self.type_astrahus,
             )
 
         with patch(
-            MODULE_PATH + ".Timer._import_schedule_notifications_for_timer"
-        ) as mock_import_func:
+            MODULE_PATH + "._task_schedule_notifications_for_timer"
+        ) as mock_schedule_notifications:
             timer.date = now() + timedelta(hours=3)
             timer.save()
-            self.assertTrue(mock_import_func.called)
-            _, kwargs = mock_import_func.return_value.apply_async.call_args
+            self.assertTrue(mock_schedule_notifications.called)
+            _, kwargs = mock_schedule_notifications.return_value.apply_async.call_args
             self.assertEqual(kwargs["kwargs"]["timer_pk"], timer.pk)
 
     def test_dont_schedule_notifications_else(self):
         with patch(
-            MODULE_PATH + ".Timer._import_schedule_notifications_for_timer"
-        ) as mock_import_func:
-            timer = Timer.objects.create(
+            MODULE_PATH + "._task_schedule_notifications_for_timer"
+        ) as mock_schedule_notifications:
+            timer = create_fake_timer(
                 date=now() + timedelta(hours=4),
                 eve_solar_system=self.system_abune,
                 structure_type=self.type_astrahus,
             )
 
         with patch(
-            MODULE_PATH + ".Timer._import_schedule_notifications_for_timer"
-        ) as mock_import_func:
+            MODULE_PATH + "._task_schedule_notifications_for_timer"
+        ) as mock_schedule_notifications:
             timer.date = now() + timedelta(hours=3)
             timer.structure_name = "Some fancy name"
-            self.assertFalse(mock_import_func.called)
+            self.assertFalse(mock_schedule_notifications.called)
+
+
+@patch(MODULE_PATH + "._task_schedule_notifications_for_timer", Mock)
+class TestTimerSaveXCalcDistances(LoadTestDataMixin, NoSocketsTestCase):
+    @classmethod
+    def setUpClass(cls) -> None:
+        super().setUpClass()
+        cls.webhook = DiscordWebhook.objects.create(
+            name="Dummy", url="http://www.example.com"
+        )
+
+    @patch(MODULE_PATH + "._task_calc_timer_distances_for_all_staging_systems")
+    def test_should_calc_distances_when_created(self, mock_calc_distances):
+        # when
+        timer = Timer.objects.create(
+            date=now() + timedelta(hours=4),
+            eve_solar_system=self.system_abune,
+            structure_type=self.type_astrahus,
+        )
+        # then
+        self.assertTrue(mock_calc_distances.called)
+        _, kwargs = mock_calc_distances.return_value.apply_async.call_args
+        self.assertEqual(kwargs["args"][0], timer.pk)
+
+    @patch(MODULE_PATH + "._task_calc_timer_distances_for_all_staging_systems")
+    def test_should_recalc_distances_when_solar_system_has_changed(
+        self, mock_calc_distances
+    ):
+        timer = create_fake_timer(
+            date=now() + timedelta(hours=4),
+            eve_solar_system=self.system_abune,
+            structure_type=self.type_astrahus,
+        )
+        # when
+        timer.eve_solar_system = self.system_enaluri
+        timer.save()
+        # then
+        self.assertTrue(mock_calc_distances.called)
+
+    @patch(MODULE_PATH + "._task_calc_timer_distances_for_all_staging_systems")
+    def test_should_not_recalc_distances_when_solar_system_unchanged(
+        self, mock_calc_distances
+    ):
+        timer = create_fake_timer(
+            date=now() + timedelta(hours=4),
+            eve_solar_system=self.system_abune,
+            structure_type=self.type_astrahus,
+        )
+        # when
+        timer.structure_type = self.type_raitaru
+        timer.save()
+        # then
+        self.assertFalse(mock_calc_distances.called)
 
 
 @patch(MODULE_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
-class TestTimerAccess(LoadTestDataMixin, TestCase):
+class TestTimerAccess(LoadTestDataMixin, NoSocketsTestCase):
     @classmethod
     def setUpClass(cls) -> None:
         super().setUpClass()
@@ -221,7 +285,7 @@ class TestTimerAccess(LoadTestDataMixin, TestCase):
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
             eve_corporation=self.corporation_1,
-            visibility=Timer.VISIBILITY_CORPORATION,
+            visibility=Timer.Visibility.CORPORATION,
             user=self.user_1,
         )
         self.assertFalse(timer.user_can_view(self.user_3))
@@ -232,7 +296,7 @@ class TestTimerAccess(LoadTestDataMixin, TestCase):
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
             eve_corporation=self.corporation_1,
-            visibility=Timer.VISIBILITY_CORPORATION,
+            visibility=Timer.Visibility.CORPORATION,
             user=self.user_1,
         )
         self.assertTrue(timer.user_can_view(self.user_2))
@@ -243,7 +307,7 @@ class TestTimerAccess(LoadTestDataMixin, TestCase):
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
             eve_alliance=self.alliance_1,
-            visibility=Timer.VISIBILITY_ALLIANCE,
+            visibility=Timer.Visibility.ALLIANCE,
             user=self.user_1,
         )
         self.assertFalse(timer.user_can_view(self.user_3))
@@ -272,16 +336,16 @@ class TestTimerAccess(LoadTestDataMixin, TestCase):
 
 @patch(MODULE_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
 @patch("structuretimers.managers.STRUCTURETIMERS_TIMERS_OBSOLETE_AFTER_DAYS", 1)
-class TestTimerManger(LoadTestDataMixin, TestCase):
+class TestTimerManger(LoadTestDataMixin, NoSocketsTestCase):
     def test_delete_old_timer(self):
-        timer_1 = Timer.objects.create(
-            timer_type=Timer.TYPE_ARMOR,
+        timer_1 = create_fake_timer(
+            timer_type=Timer.Type.ARMOR,
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
             date=now(),
         )
-        timer_2 = Timer.objects.create(
-            timer_type=Timer.TYPE_ARMOR,
+        timer_2 = create_fake_timer(
+            timer_type=Timer.Type.ARMOR,
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
             date=now() - timedelta(days=1, seconds=1),
@@ -293,7 +357,7 @@ class TestTimerManger(LoadTestDataMixin, TestCase):
 
 
 @patch(MODULE_PATH + ".DiscordWebhook.send_message", spec=True)
-class TestTimerSendNotification(LoadTestDataMixin, TestCase):
+class TestTimerSendNotification(LoadTestDataMixin, NoSocketsTestCase):
     @classmethod
     def setUpClass(cls) -> None:
         super().setUpClass()
@@ -301,20 +365,42 @@ class TestTimerSendNotification(LoadTestDataMixin, TestCase):
             name="Dummy", url="http://www.example.com"
         )
 
-    def test_minmal(self, mock_send_message):
+    @patch(MODULE_PATH + ".STRUCTURETIMER_NOTIFICATION_SET_AVATAR", True)
+    def test_should_send_minimal_notification(self, mock_send_message):
+        # given
         timer = Timer(
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
             date=now(),
         )
+        # when
         timer.send_notification(self.webhook)
-
+        # then
         self.assertEqual(mock_send_message.call_count, 1)
+        _, kwargs = mock_send_message.call_args
+        self.assertEqual(kwargs["username"], __title__)
+        self.assertIsNotNone(kwargs["avatar_url"])
+
+    @patch(MODULE_PATH + ".STRUCTURETIMER_NOTIFICATION_SET_AVATAR", False)
+    def test_should_send_notification_without_avatar(self, mock_send_message):
+        # given
+        timer = Timer(
+            eve_solar_system=self.system_abune,
+            structure_type=self.type_raitaru,
+            date=now(),
+        )
+        # when
+        timer.send_notification(self.webhook)
+        # then
+        self.assertEqual(mock_send_message.call_count, 1)
+        _, kwargs = mock_send_message.call_args
+        self.assertIsNone(kwargs["username"])
+        self.assertIsNone(kwargs["avatar_url"])
 
     def test_with_content(self, mock_send_message):
         timer = Timer(
             structure_name="Test",
-            timer_type=Timer.TYPE_ARMOR,
+            timer_type=Timer.Type.ARMOR,
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
             date=now(),
@@ -328,11 +414,11 @@ class TestTimerSendNotification(LoadTestDataMixin, TestCase):
     def test_timer_with_options_1(self, mock_send_message):
         timer = Timer(
             structure_name="Test",
-            timer_type=Timer.TYPE_ARMOR,
+            timer_type=Timer.Type.ARMOR,
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
             date=now(),
-            objective=Timer.OBJECTIVE_FRIENDLY,
+            objective=Timer.Objective.FRIENDLY,
         )
         timer.send_notification(self.webhook)
 
@@ -341,11 +427,11 @@ class TestTimerSendNotification(LoadTestDataMixin, TestCase):
     def test_timer_with_options_2(self, mock_send_message):
         timer = Timer(
             structure_name="Test",
-            timer_type=Timer.TYPE_ARMOR,
+            timer_type=Timer.Type.ARMOR,
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
             date=now(),
-            objective=Timer.OBJECTIVE_HOSTILE,
+            objective=Timer.Objective.HOSTILE,
         )
         timer.send_notification(self.webhook)
 
@@ -353,28 +439,28 @@ class TestTimerSendNotification(LoadTestDataMixin, TestCase):
 
 
 @patch(MODULE_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
-class TestTimerQuerySet(LoadTestDataMixin, TestCase):
+class TestTimerQuerySet(LoadTestDataMixin, NoSocketsTestCase):
     @patch(MODULE_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
     def setUp(self) -> None:
-        self.timer_1 = Timer.objects.create(
+        self.timer_1 = create_fake_timer(
             structure_name="Timer 1",
             date=now() + timedelta(hours=4),
             eve_character=self.character_1,
             eve_corporation=self.corporation_1,
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
-            timer_type=Timer.TYPE_ARMOR,
-            objective=Timer.OBJECTIVE_FRIENDLY,
+            timer_type=Timer.Type.ARMOR,
+            objective=Timer.Objective.FRIENDLY,
         )
-        self.timer_2 = Timer.objects.create(
+        self.timer_2 = create_fake_timer(
             structure_name="Timer 2",
             date=now() - timedelta(hours=8),
             eve_character=self.character_1,
             eve_corporation=self.corporation_1,
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
-            timer_type=Timer.TYPE_HULL,
-            objective=Timer.OBJECTIVE_FRIENDLY,
+            timer_type=Timer.Type.HULL,
+            objective=Timer.Objective.FRIENDLY,
         )
         self.timer_qs = Timer.objects.all()
         self.webhook = DiscordWebhook.objects.create(name="Dummy", url="my-url")
@@ -386,9 +472,9 @@ class TestTimerQuerySet(LoadTestDataMixin, TestCase):
         then qs contains only conforming timer
         """
         rule = NotificationRule.objects.create(
-            trigger=NotificationRule.TRIGGER_SCHEDULED_TIME_REACHED,
+            trigger=NotificationRule.Trigger.SCHEDULED_TIME_REACHED,
             scheduled_time=NotificationRule.MINUTES_10,
-            require_timer_types=[Timer.TYPE_ARMOR],
+            require_timer_types=[Timer.Type.ARMOR],
             webhook=self.webhook,
         )
         new_qs = self.timer_qs.conforms_with_notification_rule(rule)
@@ -402,7 +488,7 @@ class TestTimerQuerySet(LoadTestDataMixin, TestCase):
         then qs is empty
         """
         rule = NotificationRule.objects.create(
-            trigger=NotificationRule.TRIGGER_SCHEDULED_TIME_REACHED,
+            trigger=NotificationRule.Trigger.SCHEDULED_TIME_REACHED,
             scheduled_time=NotificationRule.MINUTES_10,
             webhook=self.webhook,
         )
@@ -418,9 +504,9 @@ class TestTimerQuerySet(LoadTestDataMixin, TestCase):
         then qs contains all timers
         """
         rule = NotificationRule.objects.create(
-            trigger=NotificationRule.TRIGGER_SCHEDULED_TIME_REACHED,
+            trigger=NotificationRule.Trigger.SCHEDULED_TIME_REACHED,
             scheduled_time=NotificationRule.MINUTES_10,
-            require_objectives=[Timer.OBJECTIVE_FRIENDLY],
+            require_objectives=[Timer.Objective.FRIENDLY],
             webhook=self.webhook,
         )
         new_qs = self.timer_qs.conforms_with_notification_rule(rule)
@@ -614,57 +700,58 @@ class TestDiscordWebhookSendMessageToWebhook(NoSocketsTestCase):
         self.assertTrue(mock_logger.warning.called)
 
 
+@patch(MODULE_PATH + "._task_calc_timer_distances_for_all_staging_systems", Mock())
 @patch(MODULE_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
-class TestNotificationRuleIsMatchingTimer(LoadTestDataMixin, TestCase):
+class TestNotificationRuleIsMatchingTimer(LoadTestDataMixin, NoSocketsTestCase):
     @patch(MODULE_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
     def setUp(self) -> None:
         self.webhook = DiscordWebhook.objects.create(name="Dummy", url="my-url")
-        self.timer = Timer.objects.create(
+        self.timer = create_fake_timer(
             structure_name="Test",
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
             date=now(),
         )
         self.rule = NotificationRule.objects.create(
-            trigger=NotificationRule.TRIGGER_SCHEDULED_TIME_REACHED,
+            trigger=NotificationRule.Trigger.SCHEDULED_TIME_REACHED,
             scheduled_time=NotificationRule.MINUTES_15,
             webhook=self.webhook,
         )
 
     def test_require_timer_types(self):
         # do not process if it does not match
-        self.rule.require_timer_types = [Timer.TYPE_ARMOR]
+        self.rule.require_timer_types = [Timer.Type.ARMOR]
         self.assertFalse(self.rule.is_matching_timer(self.timer))
 
         # process if it does match
-        self.timer.timer_type = Timer.TYPE_ARMOR
+        self.timer.timer_type = Timer.Type.ARMOR
         self.assertTrue(self.rule.is_matching_timer(self.timer))
 
     def test_exclude_timer_types(self):
         # process if it does match
-        self.rule.exclude_timer_types = [Timer.TYPE_ARMOR]
+        self.rule.exclude_timer_types = [Timer.Type.ARMOR]
         self.assertTrue(self.rule.is_matching_timer(self.timer))
 
         # do not process if it does not match
-        self.timer.timer_type = Timer.TYPE_ARMOR
+        self.timer.timer_type = Timer.Type.ARMOR
         self.assertFalse(self.rule.is_matching_timer(self.timer))
 
     def test_require_objectives(self):
         # do not process if it does not match
-        self.rule.require_objectives = [Timer.OBJECTIVE_HOSTILE]
+        self.rule.require_objectives = [Timer.Objective.HOSTILE]
         self.assertFalse(self.rule.is_matching_timer(self.timer))
 
         # process if it does match
-        self.timer.objective = Timer.OBJECTIVE_HOSTILE
+        self.timer.objective = Timer.Objective.HOSTILE
         self.assertTrue(self.rule.is_matching_timer(self.timer))
 
     def test_exclude_objectives(self):
         # process if it does match
-        self.rule.exclude_objectives = [Timer.OBJECTIVE_HOSTILE]
+        self.rule.exclude_objectives = [Timer.Objective.HOSTILE]
         self.assertTrue(self.rule.is_matching_timer(self.timer))
 
         # do not process if it does not match
-        self.timer.objective = Timer.OBJECTIVE_HOSTILE
+        self.timer.objective = Timer.Objective.HOSTILE
         self.assertFalse(self.rule.is_matching_timer(self.timer))
 
     def test_require_corporations(self):
@@ -713,25 +800,25 @@ class TestNotificationRuleIsMatchingTimer(LoadTestDataMixin, TestCase):
 
     def test_require_visibility(self):
         # do not process if it does not match
-        self.rule.require_visibility = [Timer.VISIBILITY_CORPORATION]
+        self.rule.require_visibility = [Timer.Visibility.CORPORATION]
         self.assertFalse(self.rule.is_matching_timer(self.timer))
 
         # process if it does match
-        self.timer.visibility = Timer.VISIBILITY_CORPORATION
+        self.timer.visibility = Timer.Visibility.CORPORATION
         self.assertTrue(self.rule.is_matching_timer(self.timer))
 
     def test_exclude_visibility(self):
         # process if it does match
-        self.rule.exclude_visibility = [Timer.VISIBILITY_CORPORATION]
+        self.rule.exclude_visibility = [Timer.Visibility.CORPORATION]
         self.assertTrue(self.rule.is_matching_timer(self.timer))
 
         # do not process if it does not match
-        self.timer.visibility = Timer.VISIBILITY_CORPORATION
+        self.timer.visibility = Timer.Visibility.CORPORATION
         self.assertFalse(self.rule.is_matching_timer(self.timer))
 
     def test_require_important(self):
         # do not process if it does not match
-        self.rule.is_important = NotificationRule.CLAUSE_REQUIRED
+        self.rule.is_important = NotificationRule.Clause.REQUIRED
         self.assertFalse(self.rule.is_matching_timer(self.timer))
 
         # process if it does match
@@ -740,7 +827,7 @@ class TestNotificationRuleIsMatchingTimer(LoadTestDataMixin, TestCase):
 
     def test_exclude_important(self):
         # process if it does match
-        self.rule.is_important = NotificationRule.CLAUSE_EXCLUDED
+        self.rule.is_important = NotificationRule.Clause.EXCLUDED
         self.assertTrue(self.rule.is_matching_timer(self.timer))
 
         # do not process if it does not match
@@ -749,7 +836,7 @@ class TestNotificationRuleIsMatchingTimer(LoadTestDataMixin, TestCase):
 
     def test_require_opsec(self):
         # do not process if it does not match
-        self.rule.is_opsec = NotificationRule.CLAUSE_REQUIRED
+        self.rule.is_opsec = NotificationRule.Clause.REQUIRED
         self.assertFalse(self.rule.is_matching_timer(self.timer))
 
         # process if it does match
@@ -758,7 +845,7 @@ class TestNotificationRuleIsMatchingTimer(LoadTestDataMixin, TestCase):
 
     def test_exclude_opsec(self):
         # process if it does match
-        self.rule.is_opsec = NotificationRule.CLAUSE_EXCLUDED
+        self.rule.is_opsec = NotificationRule.Clause.EXCLUDED
         self.assertTrue(self.rule.is_matching_timer(self.timer))
 
         # do not process if it does not match
@@ -767,20 +854,20 @@ class TestNotificationRuleIsMatchingTimer(LoadTestDataMixin, TestCase):
 
 
 @patch(MODULE_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
-class TestNotificationRuleQuerySet(LoadTestDataMixin, TestCase):
+class TestNotificationRuleQuerySet(LoadTestDataMixin, NoSocketsTestCase):
     @patch(MODULE_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
     def setUp(self) -> None:
         self.webhook = DiscordWebhook.objects.create(name="Dummy", url="my-url")
         self.rule_1 = NotificationRule.objects.create(
-            trigger=NotificationRule.TRIGGER_SCHEDULED_TIME_REACHED,
+            trigger=NotificationRule.Trigger.SCHEDULED_TIME_REACHED,
             scheduled_time=10,
-            require_timer_types=[Timer.TYPE_ARMOR],
+            require_timer_types=[Timer.Type.ARMOR],
             webhook=self.webhook,
         )
         self.rule_2 = NotificationRule.objects.create(
-            trigger=NotificationRule.TRIGGER_SCHEDULED_TIME_REACHED,
+            trigger=NotificationRule.Trigger.SCHEDULED_TIME_REACHED,
             scheduled_time=15,
-            require_objectives=[Timer.OBJECTIVE_FRIENDLY],
+            require_objectives=[Timer.Objective.FRIENDLY],
             webhook=self.webhook,
         )
         self.rule_qs = NotificationRule.objects.all()
@@ -791,15 +878,15 @@ class TestNotificationRuleQuerySet(LoadTestDataMixin, TestCase):
         when one rule conforms with timer
         then qs contains only conforming rule
         """
-        timer = Timer.objects.create(
+        timer = create_fake_timer(
             structure_name="Test Timer",
             date=now() + timedelta(hours=4),
             eve_character=self.character_1,
             eve_corporation=self.corporation_1,
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
-            timer_type=Timer.TYPE_ARMOR,
-            objective=Timer.OBJECTIVE_HOSTILE,
+            timer_type=Timer.Type.ARMOR,
+            objective=Timer.Objective.HOSTILE,
         )
         new_qs = self.rule_qs.conforms_with_timer(timer)
         self.assertIsInstance(new_qs, models.QuerySet)
@@ -811,15 +898,15 @@ class TestNotificationRuleQuerySet(LoadTestDataMixin, TestCase):
         when no rule conforms with timer
         then qs is empty
         """
-        timer = Timer.objects.create(
+        timer = create_fake_timer(
             structure_name="Test Timer",
             date=now() + timedelta(hours=4),
             eve_character=self.character_1,
             eve_corporation=self.corporation_1,
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
-            timer_type=Timer.TYPE_HULL,
-            objective=Timer.OBJECTIVE_HOSTILE,
+            timer_type=Timer.Type.HULL,
+            objective=Timer.Objective.HOSTILE,
         )
         new_qs = self.rule_qs.conforms_with_timer(timer)
         self.assertIsInstance(new_qs, models.QuerySet)
@@ -831,15 +918,15 @@ class TestNotificationRuleQuerySet(LoadTestDataMixin, TestCase):
         when one rule conforms with timer
         then qs contains only conforming rule
         """
-        timer = Timer.objects.create(
+        timer = create_fake_timer(
             structure_name="Test Timer",
             date=now() + timedelta(hours=4),
             eve_character=self.character_1,
             eve_corporation=self.corporation_1,
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
-            timer_type=Timer.TYPE_ARMOR,
-            objective=Timer.OBJECTIVE_FRIENDLY,
+            timer_type=Timer.Type.ARMOR,
+            objective=Timer.Objective.FRIENDLY,
         )
         new_qs = self.rule_qs.conforms_with_timer(timer)
         self.assertIsInstance(new_qs, models.QuerySet)
@@ -849,70 +936,70 @@ class TestNotificationRuleQuerySet(LoadTestDataMixin, TestCase):
 
 
 @patch(MODULE_PATH + ".NotificationRule._import_schedule_notifications_for_rule")
-class TestNotificationRuleSave(LoadTestDataMixin, TestCase):
+class TestNotificationRuleSave(LoadTestDataMixin, NoSocketsTestCase):
     @classmethod
     def setUpClass(cls) -> None:
         super().setUpClass()
         cls.webhook = DiscordWebhook.objects.create(name="dummy", url="dummy-url")
 
     @patch(MODULE_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", True)
-    def test_scheduled_normal(self, mock_import_func):
+    def test_scheduled_normal(self, mock_schedule_notifications):
         """
         given notifications are enabled
         when trigger is scheduled and enabled
         then schedule notifications
         """
         rule = NotificationRule(
-            trigger=NotificationRule.TRIGGER_SCHEDULED_TIME_REACHED,
+            trigger=NotificationRule.Trigger.SCHEDULED_TIME_REACHED,
             scheduled_time=NotificationRule.MINUTES_10,
             webhook=self.webhook,
         )
         rule.save()
-        self.assertTrue(mock_import_func.called)
+        self.assertTrue(mock_schedule_notifications.called)
 
     @patch(MODULE_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
-    def test_scheduled_disabled_1(self, mock_import_func):
+    def test_scheduled_disabled_1(self, mock_schedule_notifications):
         """
         given notifications are disabled
         when trigger is scheduled and enabled
         then do not schedule notifications
         """
         rule = NotificationRule(
-            trigger=NotificationRule.TRIGGER_SCHEDULED_TIME_REACHED,
+            trigger=NotificationRule.Trigger.SCHEDULED_TIME_REACHED,
             scheduled_time=NotificationRule.MINUTES_10,
             webhook=self.webhook,
         )
         rule.save()
-        self.assertFalse(mock_import_func.called)
+        self.assertFalse(mock_schedule_notifications.called)
 
     @patch(MODULE_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", True)
-    def test_scheduled_disabled_2(self, mock_import_func):
+    def test_scheduled_disabled_2(self, mock_schedule_notifications):
         """
         given notifications are enabled
         when trigger is scheduled and disabled
         then do not schedule notifications
         """
         rule = NotificationRule(
-            trigger=NotificationRule.TRIGGER_SCHEDULED_TIME_REACHED,
+            trigger=NotificationRule.Trigger.SCHEDULED_TIME_REACHED,
             scheduled_time=NotificationRule.MINUTES_10,
             webhook=self.webhook,
             is_enabled=False,
         )
         rule.save()
-        self.assertFalse(mock_import_func.called)
+        self.assertFalse(mock_schedule_notifications.called)
 
     @patch(MODULE_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
-    def test_created_trigger(self, mock_import_func):
+    def test_created_trigger(self, mock_schedule_notifications):
         """
         when trigger is created
         then delete all scheduled notifications based on same rule
         """
         rule = NotificationRule.objects.create(
-            trigger=NotificationRule.TRIGGER_SCHEDULED_TIME_REACHED,
+            trigger=NotificationRule.Trigger.SCHEDULED_TIME_REACHED,
             scheduled_time=NotificationRule.MINUTES_10,
             webhook=self.webhook,
         )
-        timer = Timer.objects.create(
+        timer = create_fake_timer(
             date=now() + timedelta(hours=4),
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
@@ -923,14 +1010,14 @@ class TestNotificationRuleSave(LoadTestDataMixin, TestCase):
             timer_date=timer.date,
             notification_date=timer.date - timedelta(minutes=10),
         )
-        rule.trigger = NotificationRule.TRIGGER_NEW_TIMER_CREATED
+        rule.trigger = NotificationRule.Trigger.NEW_TIMER_CREATED
         rule.scheduled_time = None
         rule.save()
 
         self.assertFalse(ScheduledNotification.objects.filter(pk=obj.pk).exists())
 
 
-class TestNotificationRuleMultiselectDisplay(TestCase):
+class TestNotificationRuleMultiselectDisplay(NoSocketsTestCase):
     @classmethod
     def setUpClass(cls) -> None:
         super().setUpClass()
@@ -950,3 +1037,49 @@ class TestNotificationRuleMultiselectDisplay(TestCase):
     def test_raises_exception_if_not_found(self):
         with self.assertRaises(ValueError):
             NotificationRule.get_multiselect_display(3, self.choices)
+
+
+@patch("structuretimers.tasks.retry_task_if_esi_is_down", lambda self: None)
+@patch(MODULE_PATH + ".EveSolarSystem.distance_to", lambda *args, **kwargs: 4.257e16)
+@patch(MODULE_PATH + ".EveSolarSystem.jumps_to", lambda *args, **kwargs: 3)
+@patch(MODULE_PATH + "._task_calc_staging_system", wraps=_task_calc_staging_system)
+@override_settings(CELERY_ALWAYS_EAGER=True)
+class TestStagingSystem(LoadTestDataMixin, NoSocketsTestCase):
+    def test_should_calc_distances(self, spy_task_calc_staging_system):
+        # given
+        timer = create_fake_timer(
+            structure_name="Test",
+            timer_type=Timer.Type.ARMOR,
+            eve_solar_system=self.system_abune,
+            structure_type=self.type_raitaru,
+            date=datetime(2020, 8, 6, 13, 25),
+        )
+        # when
+        staging_system = StagingSystem.objects.create(
+            eve_solar_system=self.system_enaluri
+        )
+        # then
+        obj = timer.distances.first()
+        self.assertEqual(obj.staging_system, staging_system)
+        self.assertAlmostEqual(obj.light_years, 4.5, delta=0.1)
+        self.assertEqual(obj.jumps, 3)
+        self.assertTrue(spy_task_calc_staging_system.called)
+
+    def test_should_not_update_distances_when_solar_system_not_changed(
+        self, spy_task_calc_staging_system
+    ):
+        # given
+        create_fake_timer(
+            structure_name="Test",
+            timer_type=Timer.Type.ARMOR,
+            eve_solar_system=self.system_abune,
+            structure_type=self.type_raitaru,
+            date=datetime(2020, 8, 6, 13, 25),
+        )
+        staging_system = create_fake_staging_system(
+            eve_solar_system=self.system_enaluri
+        )
+        # when
+        staging_system.save()
+        # then
+        self.assertFalse(spy_task_calc_staging_system.called)
